@@ -123,6 +123,8 @@ Options:
          Increment the Debian release number for a Debian Security Team upload
   -U, --upstream
          Increment the Debian release number without any distribution name
+  --team
+         Increment the Debian release number for a team upload
   --bpo
          Increment the Debian release number for a Backports.org upload
 	 to "lenny-backports"
@@ -162,6 +164,10 @@ Options:
   --[no]multimaint
          When appending an entry to a changelog section (-a), [do not]
          indicate if multiple maintainers are now involved (default: do so)
+  --[no]multimaint-merge
+         When appending an entry to a changelog section, [do not] merge the
+         entry into an existing changelog section for the current author.
+         (default: do not)
   -m, --maintmaint
          Don\'t change (maintain) the maintainer details in the changelog entry
   -t, --mainttrailer
@@ -186,7 +192,7 @@ Options:
          Display this help message and exit
   --version
          Display version information
-  At most one of -a, -i, -e, -r, -v, -d, -n, --bin-nmu, -R, -q, --qa, -s, --bpo, -l
+  At most one of -a, -i, -e, -r, -v, -d, -n, --bin-nmu, -R, -q, --qa, -s, --team, --bpo, -l
   (or their long equivalents) may be used.
   With no options, one of -i or -a is chosen by looking for a .upload
   file in the parent directory and checking its contents.
@@ -304,9 +310,9 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 # with older debchange versions.
 my ($opt_help, $opt_version);
 my ($opt_i, $opt_a, $opt_e, $opt_r, $opt_v, $opt_b, $opt_d, $opt_D, $opt_u, $opt_force_dist);
-my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_create, $opt_package, @closes);
+my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_team, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_create, $opt_package, @closes);
 my ($opt_news);
-my ($opt_ignore, $opt_level, $opt_regex, $opt_noconf, $opt_empty);
+my ($opt_level, $opt_regex, $opt_noconf, $opt_empty);
 my ($opt_distributor);
 
 Getopt::Long::Configure('bundling');
@@ -334,6 +340,7 @@ GetOptions("help|h" => \$opt_help,
 	   "q|qa" => \$opt_qa,
 	   "s|security" => \$opt_s,
 	   "U|upstream" => \$opt_U,
+	   "team" => \$opt_team,
 	   "bpo" => \$opt_bpo,
 	   "l|local=s" => \$opt_l,
 	   "query!" => \$opt_query,
@@ -342,9 +349,10 @@ GetOptions("help|h" => \$opt_help,
 	   "news:s" => \$opt_news,
 	   "multimaint!" => \$opt_multimaint,
 	   "multi-maint!" => \$opt_multimaint,
+	   'multimaint-merge!' => \$opt_multimaint_merge,
+	   'multi-maint-merge!' => \$opt_multimaint_merge,
 	   "m|maintmaint" => \$opt_m,
 	   "t|mainttrailer!" => \$opt_t,
-	   "ignore-dirname" => \$opt_ignore,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
 	   "noconf" => \$opt_noconf,
@@ -373,11 +381,6 @@ if ($opt_noconf) {
 if ($opt_help) { usage; exit 0; }
 if ($opt_version) { version; exit 0; }
 
-# dirname stuff
-if ($opt_ignore) {
-    fatal "--ignore-dirname has been replaced by --check-dirname-level and\n--check-dirname-regex; run $progname --help for more details";
-}
-
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
@@ -388,8 +391,8 @@ if (defined $opt_level) {
 if (defined $opt_regex) { $check_dirname_regex = $opt_regex; }
 
 # Only allow at most one non-help option
-fatal "Only one of -a, -i, -e, -r, -v, -d, -n/--nmu, --bin-nmu, -R/--rebuild, -q/--qa, -s/--security, --bpo, -l/--local is allowed;\ntry $progname --help for more help"
-    if ($opt_i?1:0) + ($opt_a?1:0) + ($opt_e?1:0) + ($opt_r?1:0) + ($opt_v?1:0) + ($opt_d?1:0) + ($opt_n?1:0) + ($opt_bn?1:0) + ($opt_R?1:0) + ($opt_qa?1:0) + ($opt_s?1:0) + ($opt_bpo?1:0) + ($opt_l?1:0) > 1;
+fatal "Only one of -a, -i, -e, -r, -v, -d, -n/--nmu, --bin-nmu, -R/--rebuild, -q/--qa, -s/--security, --team, --bpo, -l/--local is allowed;\ntry $progname --help for more help"
+    if ($opt_i?1:0) + ($opt_a?1:0) + ($opt_e?1:0) + ($opt_r?1:0) + ($opt_v?1:0) + ($opt_d?1:0) + ($opt_n?1:0) + ($opt_bn?1:0) + ($opt_R?1:0) + ($opt_qa?1:0) + ($opt_s?1:0) + ($opt_team?1:0) + ($opt_bpo?1:0) + ($opt_l?1:0) > 1;
 
 if ($opt_s) {
     $opt_u = "high";
@@ -416,13 +419,13 @@ if ($distributor eq 'Ubuntu' and ($opt_n or $opt_bn or $opt_qa or $opt_bpo)) {
 # Check the distro name given.
 if (defined $opt_D) {
     if ($distributor eq 'Debian') {
-	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|(sarge|etch|lenny)(-volatile|-backports)|((oldstable|testing)-)?proposed-updates)$/) {
-	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, {sarge,etch,lenny}-{volatile,backports},\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|(etch|lenny)-volatile|lenny-backports|((oldstable|testing)-)?proposed-updates)$/) {
+	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, {etch,lenny}-volatile, lenny-backports,\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } elsif ($distributor eq 'Ubuntu') {
-	unless ($opt_D =~ /^((dapper|hardy|intrepid|jaunty|karmic|lucid|maverick)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
-	    warn "$progname warning: Recognised distributions are:\n{dapper,hardy,intrepid,jaunty,karmic,lucid,maverick}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^((dapper|hardy|jaunty|karmic|lucid|maverick|natty)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
+	    warn "$progname warning: Recognised distributions are:\n{dapper,hardy,jaunty,karmic,lucid,maverick,natty}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } else {
@@ -450,8 +453,8 @@ fatal "--package cannot be used when creating a NEWS file"
 
 if ($opt_create) {
     if ($opt_a || $opt_i || $opt_e || $opt_r || $opt_b || $opt_n || $opt_bn ||
-	    $opt_R || $opt_qa || $opt_s || $opt_bpo || $opt_l || $opt_allow_lower) {
-	warn "$progname warning: ignoring -a/-i/-e/-r/-b/--allow-lower-version/-n/--bin-nmu/-R/-q/--qa/-s/--bpo/-l options with --create\n";
+	    $opt_R || $opt_qa || $opt_s || $opt_team || $opt_bpo || $opt_l || $opt_allow_lower) {
+	warn "$progname warning: ignoring -a/-i/-e/-r/-b/--allow-lower-version/-n/--bin-nmu/-R/-q/--qa/-s/--team/--bpo/-l options with --create\n";
 	$warnings++;
     }
     if ($opt_package && $opt_d) {
@@ -565,8 +568,8 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     fatal "No distribution in changelog!"
 	unless exists $changelog{'Distribution'};
     if ($distributor eq 'Ubuntu') {
-	# In Ubuntu uploads should go to lucid
-	$DISTRIBUTION = 'lucid';
+	# In Ubuntu uploads should go to maverick
+	$DISTRIBUTION = 'maverick';
     } else {
 	$DISTRIBUTION=$changelog{'Distribution'};
     }
@@ -738,7 +741,7 @@ if (! $opt_m) {
 #####
 
 if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and 
-    ! $opt_qa and ! $opt_bpo and ! $opt_bn and ! $opt_R and ! $opt_n and ! $opt_c and
+    ! $opt_team and ! $opt_qa and ! $opt_bpo and ! $opt_bn and ! $opt_R and ! $opt_n and ! $opt_c and
     ! (exists $ENV{'CHANGELOG'} and length $ENV{'CHANGELOG'}) and
     ! $opt_create and ! $opt_a_passed and ! $opt_r and ! $opt_e and
     $distributor ne 'Ubuntu' and
@@ -756,7 +759,7 @@ if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
 	    my $packager = "$MAINTAINER <$EMAIL>";
 
 	    if (! grep { $_ eq $packager } ($maintainer, @uploaders) and
-		$packager ne $changelog{'Maintainer'}) {
+		$packager ne $changelog{'Maintainer'} and ! $opt_team) {
 		$opt_n=1;
 		$opt_a=0;
 	    }
@@ -873,7 +876,7 @@ if ($opt_news && !$opt_i && !$opt_a) {
 
 # Are we going to have to figure things out for ourselves?
 if (! $opt_i && ! $opt_v && ! $opt_d && ! $opt_a && ! $opt_e && ! $opt_r &&
-    ! $opt_n && ! $opt_bn && ! $opt_R && ! $opt_qa && ! $opt_s && ! $opt_bpo &&
+    ! $opt_n && ! $opt_bn && ! $opt_R && ! $opt_qa && ! $opt_s && ! $opt_team && ! $opt_bpo &&
     ! $opt_l && ! $opt_create) {
     # Yes, we are
     if ($opt_release_heuristic eq 'log') {
@@ -950,7 +953,7 @@ my $line;
 my $optionsok=0;
 my $merge=0;
 
-if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $opt_l || $opt_v || $opt_d ||
+if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_team || $opt_bpo || $opt_l || $opt_v || $opt_d ||
     ($opt_news && $VERSION ne $changelog{'Version'})) && ! $opt_create) {
 
     $optionsok=1;
@@ -1016,7 +1019,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
     # including epochs.
 
     if (! $NEW_VERSION) {
-	if ($VERSION =~ /(.*?)([a-yA-Y][a-zA-Z]*|\d+)(~)?$/i) {
+	if ($VERSION =~ /(.*?)([a-yA-Y][a-zA-Z]*|\d+)([+~])?$/i) {
 	    my $extra=$3 || '';
 	    my $useextra = 0;
 	    my $end=$2;
@@ -1150,6 +1153,9 @@ if (($opt_i || $opt_n || $opt_bn || $opt_R || $opt_qa || $opt_s || $opt_bpo || $
 		print O "  * Non-maintainer upload by the Security Team.\n";
 	    }
 	    $line = 1;
+	} elsif ($opt_team && ! $opt_news) {
+	    print O "  * Team upload.\n";
+	    $line = 1;
 	} elsif ($opt_bpo && ! $opt_news) {
 	    print O "  * Rebuild for $bpo_dist.\n";
 	    $line = 1;
@@ -1197,18 +1203,25 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 	    # maintainer
 	    $nextmaint ||= $1;
 	}
-	elsif (defined $lastmaint) {
-	    if (m/^\w[-+0-9a-z.]* \([^\(\) \t]+\)((?:\s+[-+0-9a-z.]+)+)\;\s+urgency=(\w+)/i) {
+	elsif (m/^\w[-+0-9a-z.]* \(([^\(\) \t]+)\)((?:\s+[-+0-9a-z.]+)+)\;\s+urgency=(\w+)/i) {
+	    if (defined $lastmaint) {
 		$lastheader = $_;
-		$lastdist = $1;
+		$lastdist = $2;
 		$lastdist =~ s/^\s+//;
 		undef $lastdist if $lastdist eq "UNRELEASED";
 		# Revert to our previously saved position
 		$line = $savedline;
 		last;
 	    }
+	    else {
+		my $tmpver = $1;
+		$tmpver =~ s/^\s+//;
+		if ($tmpver =~ m/~bpo(\d+)\+/ && exists $bpo_dists{$1}) {
+		    $dist_indicator = "$bpo_dists{$1}-backports";
+		}
+	    }
 	}
-	elsif (/  \* Upload to ([^ ]+).*$/) {
+	elsif (/  \* (?:Upload to|Rebuild for) (\S+).*$/) {
 	    ($dist_indicator = $1) =~ s/[!:.,;]$//;
 	    chomp $dist_indicator;
 	}
@@ -1267,7 +1280,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 		    $distribution = $dist_indicator;
 		} else {
 		    if ($distributor eq 'Ubuntu') {
-			$distribution = $opt_D || "lucid";
+			$distribution = $opt_D || "maverick";
 		    } else {
 			$distribution = $opt_D || $lastdist || "unstable";
 		    }
@@ -1477,7 +1490,7 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	}
 	# And check whether a new orig tarball exists
 	my @origs = glob("../$PACKAGE\_$new_uversion.*");
-	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.(gz|bz2|lzma)$/ } @origs;
+	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.(gz|bz2|lzma|xz)$/ } @origs;
 	if ($num_origs == 0) {
 	    warn "$progname warning: no orig tarball found for the new version.\n";
 	}
