@@ -25,11 +25,9 @@
 use 5.008;  # uses 'our' variables and filetest
 use strict;
 use Cwd;
-use Cwd 'abs_path';
-use Dpkg::IPC;
 use File::Basename;
 use File::Copy;
-use File::Temp qw/tempfile tempdir/;
+use File::Temp qw/tempdir/;
 use filetest 'access';
 use Getopt::Long;
 use lib '/usr/share/devscripts';
@@ -262,7 +260,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 my $debug = 0;
 my ($opt_h, $opt_v, $opt_destdir, $opt_download, $opt_force_download,
     $opt_report, $opt_passive, $opt_symlink, $opt_repack);
-my ($opt_verbose, $opt_ignore, $opt_level, $opt_regex, $opt_noconf);
+my ($opt_verbose, $opt_level, $opt_regex, $opt_noconf);
 my ($opt_package, $opt_uversion, $opt_watchfile, $opt_dehs, $opt_timeout);
 my $opt_download_version;
 my $opt_user_agent;
@@ -287,7 +285,6 @@ GetOptions("help" => \$opt_h,
 	   "dehs!" => \$opt_dehs,
 	   "verbose!" => \$opt_verbose,
 	   "debug" => \$debug,
-	   "ignore-dirname" => \$opt_ignore,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
 	   "user-agent=s" => \$opt_user_agent,
@@ -324,11 +321,6 @@ if ($dehs) {
     $SIG{'__DIE__'} = \&dehs_die;
 }
 
-# dirname stuff
-if ($opt_ignore) {
-    die "$progname: --ignore-dirname has been replaced by --check-dirname-level and\n--check-dirname-regex; run $progname --help for more details\n";
-}
-
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
@@ -529,7 +521,7 @@ for my $dir (@dirs) {
 	    print "-- Found watchfile in $dir/debian\n" if $verbose;
 	} else {
 	    print "-- Skip watchfile in $dir/debian since it does not match the package name\n" .
-	        "   (or the settings of the--check-dirname-level and --check-dirname-regex options if any).\n"
+	        "   (or the settings of the --check-dirname-level and --check-dirname-regex options if any).\n"
 	        if $verbose;
 	    next;
 	}
@@ -564,7 +556,7 @@ if (defined $opt_uversion) {
 
 # Now sort the list of directories, so that we process the most recent
 # directories first, as determined by the package version numbers
-@debdirs = Devscripts::Versort::versort(@debdirs);
+@debdirs = Devscripts::Versort::deb_versort(@debdirs);
 
 # Now process the watchfiles in order.  If a directory d has subdirectories
 # d/sd1/debian and d/sd2/debian, which each contain watchfiles corresponding
@@ -1202,7 +1194,7 @@ EOF
 
     # Can't just use $lastversion eq $newversion, as then 0.01 and 0.1
     # compare different, whereas they are treated as equal by dpkg
-    if (system("dpkg", "--compare-versions", "'$mangled_lastversion'", "eq", "'$newversion'") == 0) {
+    if (system("dpkg", "--compare-versions", "$mangled_lastversion", "eq", "$newversion") == 0) {
 	if ($verbose or ($download == 0 and $report and ! $dehs)) {
 	    print $pkg_report_header;
 	    $pkg_report_header = '';
@@ -1229,7 +1221,7 @@ EOF
     # We use dpkg's rules to determine whether our current version
     # is newer or older than the remote version.
     if (!defined $download_version) {
-	if (system("dpkg", "--compare-versions", "'$mangled_lastversion'", "gt", "'$newversion'") == 0) {
+	if (system("dpkg", "--compare-versions", "$mangled_lastversion", "gt", "$newversion") == 0) {
 	    if ($verbose) {
 		print " => remote site does not even have current version\n";
 	    } elsif ($dehs) {
@@ -1346,14 +1338,8 @@ EOF
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.bz2|tbz2?)$/) {
 	print "-- Repacking from bzip2 to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
-	my (undef, $fname) = tempfile(UNLINK => 1);
-	fork_and_exec(exec => ['bunzip2', '-c', "$destdir/$newfile_base"],
-		      to_file => $fname,
-		      wait_child => 1);
-	fork_and_exec(exec => ['gzip', '-n', '-9'],
-		      from_file => $fname,
-		      to_file => "$destdir/$newfile_base_gz",
-		      wait_child => 1);
+	system("bunzip2 -c $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
+	  or die "repacking from bzip2 to gzip failed\n";
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1361,14 +1347,17 @@ EOF
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.lzma|tlz(?:ma?)?)$/) {
 	print "-- Repacking from lzma to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
-	my (undef, $fname) = tempfile(UNLINK => 1);
-	fork_and_exec(exec => ['xz', '-F', 'lzma', '-cd', "$destdir/$newfile_base"],
-		      to_file => $fname,
-		      wait_child => 1);
-	fork_and_exec(exec => ['gzip', '-n', '-9'],
-		      from_file => $fname,
-		      to_file => "$destdir/$newfile_base_gz",
-		      wait_child => 1);
+	system("lzma -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
+	  or die "repacking from lzma to gzip failed\n";
+	unlink "$destdir/$newfile_base";
+	$newfile_base = $newfile_base_gz;
+    }
+
+    if ($repack and $newfile_base =~ /^(.*)\.(tar\.xz|txz)$/) {
+	print "-- Repacking from xz to gzip\n" if $verbose;
+	my $newfile_base_gz = "$1.tar.gz";
+	system("xz -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
+	  or die "repacking from xz to gzip failed\n";
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1380,97 +1369,99 @@ EOF
 	  or die("unzip binary not found. You need to install the package unzip to be able to repack .zip upstream archives.\n");
 
 	my $newfile_base_gz = "$1.tar.gz";
-	my $tempdir = tempdir ("uscanXXXX", TMPDIR => 1, CLEANUP => 1);
-	# Parent of the target directory should be under our control
-	$tempdir .= '/repack';
-	mkdir $tempdir or die("Unable to mkdir($tempdir): $!\n");
-	my $absdestdir = abs_path($destdir);
-	system('unzip', '-q', '-a', '-d', $tempdir, "$destdir/$newfile_base") == 0
-	    or die("Repacking from zip to tar.gz failed (could not unzip)\n");
-	my $cwd = cwd();
-	chdir($tempdir) or die("Unable to chdir($tempdir): $!\n");
-	eval {
-	    fork_and_exec(exec => ['tar', '--owner=root', '--group=root', '--mode=a+rX', '-czf', "$absdestdir/$newfile_base_gz", glob('* .[!.]*')],
-			  env => { GZIP => '-n -9' },
-			  wait_child => 1);
-	};
-	if ($@) {
-	    die("Repacking from zip to tar.gz failed (could not create tarball)\n");
-	}
-	chdir($cwd);
+	my $tempdir = tempdir ( "uscanXXXX", TMPDIR => 1, CLEANUP => 1 );
+	system("unzip -q -d $tempdir $destdir/$newfile_base; GZIP=-9 tar -C $tempdir -czf $destdir/$newfile_base_gz .") == 0 
+	  or die("Repacking from zip to tar.gz failed\n");
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
 
-    if ($newfile_base =~ /\.(tar\.gz|tgz
-			     |tar\.bz2|tbz2?
-			     |tar.lzma|tlz(?:ma?)?)$/x) {
-	my $filetype;
-	eval {
-	    fork_and_exec(exec => ['file', '-b', '-k', "$destdir/$newfile_base"],
-			  to_string => \$filetype,
-			  wait_child => 1);
-	};
-	unless (defined $filetype && $filetype =~ /compressed data/) {
+    if ($newfile_base =~ /\.(tar\.gz|tgz|tar\.bz2|tbz2?)$/) {
+	my $filetype = `file $destdir/$newfile_base`;
+	$filetype =~ s%^\.\./\Q$newfile_base\E: %%;
+	unless ($filetype =~ /compressed data/) {
 	    warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
 	    return 1;
 	}
     }
 
-    my @renames = (
-	[qr/\.(tar\.gz|tgz)$/, 'gz'],
-	[qr/\.(tar\.bz2|tbz2?)$/, 'bz2'],
-	[qr/\.tar\.lzma|tlz(?:ma?)?$/, 'lzma'],
-    );
-
-    my ($renamed_base);
-    foreach my $pair (@renames) {
-	if ($newfile_base !~ $pair->[0]) {
-	    next;
-	}
-
-	my ($pattern, $suffix) = @{$pair};
-	$renamed_base = "${pkg}_${newversion}.orig.tar.$suffix";
+    if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
 	if ($symlink eq 'symlink') {
-	    symlink $newfile_base, "$destdir/$renamed_base";
+	    symlink $newfile_base, "$destdir/${pkg}_${newversion}.orig.tar.gz";
 	} elsif ($symlink eq 'rename') {
-	    move "$destdir/$newfile_base", "$destdir/$renamed_base";
+	    move "$destdir/$newfile_base", "$destdir/${pkg}_${newversion}.orig.tar.gz";
 	}
-	if ($verbose) {
-	    print "-- Successfully downloaded updated package $newfile_base\n";
+    }
+    elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	if ($symlink eq 'symlink') {
+	    symlink $newfile_base, "$destdir/${pkg}_${newversion}.orig.tar.bz2";
+	} elsif ($symlink eq 'rename') {
+	    move "$destdir/$newfile_base", "$destdir/${pkg}_${newversion}.orig.tar.bz2";
+	}
+    }
+
+    if ($verbose) {
+	print "-- Successfully downloaded updated package $newfile_base\n";
+	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
 	    if ($symlink eq 'symlink') {
-		print "    and symlinked $renamed_base to it\n";
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.gz to it\n";
 	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as $renamed_base\n";
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
 	    }
-	} elsif ($dehs) {
-	    my $msg = "Successfully downloaded updated package $newfile_base";
-	    $dehs_tags{'target'} = "$renamed_base";
+	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
 	    if ($symlink eq 'symlink') {
-		$msg .= " and symlinked $renamed_base to it";
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
 	    } elsif ($symlink eq 'rename') {
-		$msg .= " and renamed it as $renamed_base";
-	    } else {
-		$dehs_tags{'target'} = $newfile_base;
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
 	    }
-	    dehs_msg($msg);
-	} else {
-	    print "$pkg: Successfully downloaded updated package $newfile_base\n";
+	}
+    } elsif ($dehs) {
+	my $msg = "Successfully downloaded updated package $newfile_base";
+	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
 	    if ($symlink eq 'symlink') {
-		print "    and symlinked $renamed_base to it\n";
+		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.gz to it";
 	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as $renamed_base\n";
+		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.gz";
+	    }
+	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    if ($symlink eq 'symlink') {
+		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it";
+	    } elsif ($symlink eq 'rename') {
+		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.bz2";
+	    }
+	}
+	dehs_msg($msg);
+    } else {
+	print "$pkg: Successfully downloaded updated package $newfile_base\n";
+	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
+	    if ($symlink eq 'symlink') {
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.gz to it\n";
+	    } elsif ($symlink eq 'rename') {
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
+	    }
+	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    if ($symlink eq 'symlink') {
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
+	    } elsif ($symlink eq 'rename') {
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
 	    }
 	}
-	last;
     }
 
     # Do whatever the user wishes to do
     if ($action) {
 	my $usefile = "$destdir/$newfile_base";
 	my @cmd = shellwords($action);
-	if ($symlink =~ /^(symlink|rename)$/ && $renamed_base) {
-	    $usefile = "$destdir/$renamed_base";
+	if ($symlink =~ /^(symlink|rename)$/
+	    and $newfile_base =~ /\.(tar\.gz|tgz)$/) {
+	    $usefile = "$destdir/${pkg}_${newversion}.orig.tar.gz";
+	}
+	elsif ($symlink =~ /^(symlink|rename)$/
+	    and $newfile_base =~ /\.(tar\.bz2|tbz2)$/) {
+	    $usefile = "$destdir/${pkg}_${newversion}.orig.tar.bz2";
 	}
 
 	# Any symlink requests are already handled by uscan
